#!/bin/sh
#   Execute a program for X, shows a message if missing
#   and helps maintain a 'last run' menu in fvwm2
#

#------------------------------------------------------------------------------
# runmsg()
#------------------------------------------------------------------------------
runmsg()
{

    umsg="$1"

    xmessage -geometry '-0+0' -buttons "TFB" "$umsg" &

    if test "$rm_lock" = "Y"
    then
	rm -f "$xrunlock"
    fi

    exit 0

} # END: runmsg()

#------------------------------------------------------------------------------
# timemsg() -- show a message that ends after 1 second
#------------------------------------------------------------------------------
timemsg()
{

    umsg="$1"

    if test "$umsg" != ""
    then
	xmessage -geometry '-0+0' -timeout 1 -buttons "OK" "$umsg" &
    fi

} # END: timemsg()

#------------------------------------------------------------------------------
# findmag() -- find a Maginifer
#------------------------------------------------------------------------------
findmag()
{
    type wmagnify > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="wmagnify"
	full_line="$prog"
	return
    fi

    type kmag > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="kmag"
	full_line="$prog"
	return
    fi

    #--- this should always exist
    type xmag > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="xmag"
	full_line="$prog"
	return
    fi

} # END: findmag()

#------------------------------------------------------------------------------
# findfm() -- find a file manager
#------------------------------------------------------------------------------
findfm()
{
    type nautilus > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="nautilus"
	full_line="$prog --no-desktop"
	return
    fi

    type thunar > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="thunar"
	full_line="$prog"
	return
    fi

    type xfe > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="xfe"
	full_line="$prog"
	return
    fi

    type xdir > /dev/null 2>&1
    if test "$?" -eq "0"
    then
	prog="xdir"
	full_line="$prog"
	return
    fi

} # END: findfm()

#------------------------------------------------------------------------------
# findprog() -- locate a standard prog
#------------------------------------------------------------------------------
findprog()
{
    case "$prog" in
	"FILEMANAGER")
	    findfm
	    ;;
	"MAGNIFY")
	    findmag
	    ;;
    esac

} # END: findprog()

#------------------------------------------------------------------------------
# mk_last() -- Regenerate last run file for FVWM, try and generate the file
#              in reverse order.  As far as I know only jtail can do that
#------------------------------------------------------------------------------
mk_last()
{
    ###debug_file=$HOME/.fvwm/DEBUG.txt
    #echo "DEBUG D10 mk_last(): $pname -- $full_line" >> "$debug_file"

    if test -d "$fwork"
    then
	last_file="$lrun"
	last_fmt="$lfmt"
	touch "$last_file" "$last_fmt"
	chmod 644 "$last_file" "$last_fmt"
	pname=`echo $full_line | awk '{print $1}'`
	grep -v ^"$pname" <  "$last_file" \
	    | sed 's/[	 ]*$//g'      \
	    | sed 's/^[	 ]*//'        \
	    | grep -v '^$'                \
	    | tail -14                    \
	    > "$last_file".OLD
	echo "$full_line" >> "$last_file".OLD
	mv "$last_file".OLD "$last_file"
	type jtail > /dev/null 2>&1
	if test "$?" -eq "0"
	then
	    #--- this reverse list entries, only nice to have
	    jtail -n 15 -r "$last_file" > "$last_fmt"
	else
	    cp "$last_file" "$last_fmt"
	fi
	chmod 644 "$last_file" "$last_fmt"
    fi

} # END: mk_last()

###############################################################################
# main
###############################################################################
full_line="$@"
prog="$1"

fhome=$HOME/.fvwm/g3
fwork=$fhome/work

lrun=$fwork/last_run_x.txt
xrunlock="$fwork/fvwm-lock.$USER.xrun_prog.lock"
lfmt=$fwork/last_run_x_fmt.txt

rm_lock="N"

type killall > /dev/null 2>&1
if test "$?" -eq "0"
then
    killprog=killall
else
    killprog=pkill
fi

#------------------------------------------------------------------------------
# some overrides, FreeBSD and Linux locations differ for some apps
#------------------------------------------------------------------------------
findprog

#------------------------------------------------------------------------------
# validate command
#------------------------------------------------------------------------------
type "$prog" > /dev/null 2>&1
err_code="$?"
if test "$err_code" -ne "0"
then
    runmsg "Program $prog cannot be located"
fi

#------------------------------------------------------------------------------
# log the command and validate lock
# lock file needed to avoid collisions
#------------------------------------------------------------------------------
hour=`date "+%Y%m%d_%H"`

if test -f "$xrunlock"
then
    now=`head -n 1 < "$xrunlock" | cut -c '1-11'`
    #--- if the lock file is old from a different hour, 
    #    process as if it does not exist
    if test "$hour" = "$now"
    then
	rm_lock="N"
    else
	rm_lock="Y"
	date "+%Y%m%d_%H%M%S" > "$xrunlock"
	echo "$full_line" >> "$xrunlock"
    fi
else
    rm_lock="Y"
    date "+%Y%m%d_%H%M%S" > "$xrunlock"
    echo "$full_line" >> "$xrunlock"
fi

if test "$rm_lock" = "Y"
then
    mk_last $full_line &
fi

#------------------------------------------------------------------------------
# process the command
#------------------------------------------------------------------------------
timemsg "STARTING: $full_line"

if test "$rm_lock" = "Y"
then
    rm -f "$xrunlock"
fi

exec $full_line &

exit 0
